@@ -16,18 +16,14 @@
 from __future__ import print_function
 
 import json
-import logging
 import sys
 
-from cliff import command
-from cliff import lister
-from cliff import show
+from osc_lib.command import command
 from osc_lib import utils
 
 import ironic_inspector_client
 
 
-LOG = logging.getLogger('ironic_inspector.shell')
 API_NAME = 'baremetal_introspection'
 API_VERSION_OPTION = 'inspector_api_version'
 DEFAULT_API_VERSION = '1'
@@ -61,14 +57,15 @@ def build_option_parser(parser):
     return parser
 
 
-class StartCommand(lister.Lister):
+class StartCommand(command.Lister):
     """Start the introspection."""
 
     COLUMNS = ('UUID', 'Error')
 
     def get_parser(self, prog_name):
         parser = super(StartCommand, self).get_parser(prog_name)
-        parser.add_argument('uuid', help='baremetal node UUID(s)', nargs='+')
+        parser.add_argument('node', help='baremetal node UUID(s) or name(s)',
+                            nargs='+')
         parser.add_argument('--new-ipmi-username',
                             default=None,
                             help='if set, *Ironic Inspector* will update IPMI '
@@ -85,7 +82,7 @@ class StartCommand(lister.Lister):
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.baremetal_introspection
-        for uuid in parsed_args.uuid:
+        for uuid in parsed_args.node:
             client.introspect(uuid,
                               new_ipmi_username=parsed_args.new_ipmi_username,
                               new_ipmi_password=parsed_args.new_ipmi_password)
@@ -95,7 +92,7 @@ class StartCommand(lister.Lister):
 
         if parsed_args.wait:
             print('Waiting for introspection to finish...', file=sys.stderr)
-            result = client.wait_for_finish(parsed_args.uuid)
+            result = client.wait_for_finish(parsed_args.node)
             result = [(uuid, s.get('error'))
                       for uuid, s in result.items()]
         else:
@@ -109,26 +106,73 @@ class ReprocessCommand(command.Command):
 
     def get_parser(self, prog_name):
         parser = super(ReprocessCommand, self).get_parser(prog_name)
-        parser.add_argument('uuid', help='baremetal node UUID')
+        parser.add_argument('node', help='baremetal node UUID or name')
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.baremetal_introspection
-        client.reprocess(parsed_args.uuid)
+        client.reprocess(parsed_args.node)
 
 
-class StatusCommand(show.ShowOne):
+class StatusCommand(command.ShowOne):
     """Get introspection status."""
+    hidden_status_items = {'links'}
+
+    @classmethod
+    def status_attributes(cls, client_item):
+        """Get status attributes from an API client dict.
+
+        Filters the status fields according to the cls.hidden_status_items
+        :param client_item: an item returned from either the get_status or the
+                            list_statuses client method
+        :return: introspection status as a list of name, value pairs
+        """
+        return [item for item in client_item.items()
+                if item[0] not in cls.hidden_status_items]
 
     def get_parser(self, prog_name):
         parser = super(StatusCommand, self).get_parser(prog_name)
-        parser.add_argument('uuid', help='baremetal node UUID')
+        parser.add_argument('node', help='baremetal node UUID or name')
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.baremetal_introspection
-        status = client.get_status(parsed_args.uuid)
-        return zip(*sorted(status.items()))
+        status = client.get_status(parsed_args.node)
+        return zip(*sorted(self.status_attributes(status)))
+
+
+class StatusListCommand(command.Lister):
+    """List introspection statuses"""
+
+    COLUMNS = ('UUID', 'Started at', 'Finished at', 'Error')
+
+    @classmethod
+    def status_row(cls, client_item):
+        """Get a row from a client_item.
+
+        The row columns are filtered&sorted according to cls.COLUMNS.
+
+        :param client_item: an item returned from either the get_status or the
+                            list_statuses client method.
+        :return: a list of client_item attributes as the row
+        """
+        status = dict(StatusCommand.status_attributes(client_item))
+        return utils.get_dict_properties(status, cls.COLUMNS)
+
+    def get_parser(self, prog_name):
+        parser = super(StatusListCommand, self).get_parser(prog_name)
+        parser.add_argument('--marker', help='UUID of the last item on the '
+                                             'previous page', default=None)
+        parser.add_argument('--limit', help='the amount of items to return',
+                            type=int, default=None)
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.baremetal_introspection
+        statuses = client.list_statuses(marker=parsed_args.marker,
+                                        limit=parsed_args.limit)
+        rows = [self.status_row(status) for status in statuses]
+        return self.COLUMNS, rows
 
 
 class AbortCommand(command.Command):
@@ -136,15 +180,15 @@ class AbortCommand(command.Command):
 
     def get_parser(self, prog_name):
         parser = super(AbortCommand, self).get_parser(prog_name)
-        parser.add_argument('uuid', help='baremetal node UUID')
+        parser.add_argument('node', help='baremetal node UUID or name')
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.baremetal_introspection
-        client.abort(parsed_args.uuid)
+        client.abort(parsed_args.node)
 
 
-class RuleImportCommand(lister.Lister):
+class RuleImportCommand(command.Lister):
     """Import one or several introspection rules from a json file."""
 
     COLUMNS = ("UUID", "Description")
@@ -169,7 +213,7 @@ class RuleImportCommand(lister.Lister):
         return self.COLUMNS, result
 
 
-class RuleListCommand(lister.Lister):
+class RuleListCommand(command.Lister):
     """List all introspection rules."""
 
     COLUMNS = ("UUID", "Description")
@@ -182,7 +226,7 @@ class RuleListCommand(lister.Lister):
         return self.COLUMNS, rules
 
 
-class RuleShowCommand(show.ShowOne):
+class RuleShowCommand(command.ShowOne):
     """Show an introspection rule."""
 
     def get_parser(self, prog_name):
@@ -226,12 +270,12 @@ class DataSaveCommand(command.Command):
         parser.add_argument("--file", metavar="<filename>",
                             help="downloaded introspection data filename "
                             "(default: stdout)")
-        parser.add_argument('uuid', help='baremetal node UUID')
+        parser.add_argument('node', help='baremetal node UUID or name')
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.baremetal_introspection
-        data = client.get_data(parsed_args.uuid,
+        data = client.get_data(parsed_args.node,
                                raw=bool(parsed_args.file))
         if parsed_args.file:
             with open(parsed_args.file, 'wb') as fp:
