@@ -26,6 +26,8 @@ from ironic_inspector.test import functional
 from oslo_concurrency import processutils
 
 import ironic_inspector_client as client
+from ironic_inspector_client.common import http
+from ironic_inspector_client import shell
 
 
 class TestV1PythonAPI(functional.Base):
@@ -34,6 +36,10 @@ class TestV1PythonAPI(functional.Base):
         self.client = client.ClientV1()
         functional.cfg.CONF.set_override('store_data', '', 'processing')
 
+    def my_status_index(self, statuses):
+        my_status = self._fake_status()
+        return statuses.index(my_status)
+
     def test_introspect_get_status(self):
         self.client.introspect(self.uuid)
         eventlet.greenthread.sleep(functional.DEFAULT_SLEEP)
@@ -41,7 +47,7 @@ class TestV1PythonAPI(functional.Base):
                                                               'reboot')
 
         status = self.client.get_status(self.uuid)
-        self.assertEqual({'finished': False, 'error': None}, status)
+        self.check_status(status, finished=False)
 
         res = self.call_continue(self.data)
         self.assertEqual({'uuid': self.uuid}, res)
@@ -52,7 +58,29 @@ class TestV1PythonAPI(functional.Base):
             node_uuid=self.uuid, address='11:22:33:44:55:66')
 
         status = self.client.get_status(self.uuid)
-        self.assertEqual({'finished': True, 'error': None}, status)
+        self.check_status(status, finished=True)
+
+    def test_introspect_list_statuses(self):
+        self.client.introspect(self.uuid)
+        eventlet.greenthread.sleep(functional.DEFAULT_SLEEP)
+        self.cli.node.set_power_state.assert_called_once_with(self.uuid,
+                                                              'reboot')
+
+        statuses = self.client.list_statuses()
+        my_status = statuses[self.my_status_index(statuses)]
+        self.check_status(my_status, finished=False)
+
+        res = self.call_continue(self.data)
+        self.assertEqual({'uuid': self.uuid}, res)
+        eventlet.greenthread.sleep(functional.DEFAULT_SLEEP)
+
+        self.assertCalledWithPatch(self.patch, self.cli.node.update)
+        self.cli.port.create.assert_called_once_with(
+            node_uuid=self.uuid, address='11:22:33:44:55:66')
+
+        statuses = self.client.list_statuses()
+        my_status = statuses[self.my_status_index(statuses)]
+        self.check_status(my_status, finished=True)
 
     def test_wait_for_finish(self):
         shared = [0]  # mutable structure to hold number of retries
@@ -71,13 +99,13 @@ class TestV1PythonAPI(functional.Base):
         eventlet.greenthread.sleep(functional.DEFAULT_SLEEP)
 
         status = self.client.get_status(self.uuid)
-        self.assertEqual({'finished': False, 'error': None}, status)
+        self.check_status(status, finished=False)
 
         self.client.wait_for_finish([self.uuid], sleep_function=fake_waiter,
                                     retry_interval=functional.DEFAULT_SLEEP)
 
         status = self.client.get_status(self.uuid)
-        self.assertEqual({'finished': True, 'error': None}, status)
+        self.check_status(status, finished=True)
 
     @mock.patch.object(swift, 'store_introspection_data', autospec=True)
     @mock.patch.object(swift, 'get_introspection_data', autospec=True)
@@ -97,16 +125,14 @@ class TestV1PythonAPI(functional.Base):
         self.cli.node.set_power_state.assert_called_once_with(self.uuid,
                                                               'reboot')
         status = self.client.get_status(self.uuid)
-        self.assertEqual({'finished': False, 'error': None},
-                         status)
+        self.check_status(status, finished=False)
 
         res = self.call_continue(self.data)
         self.assertEqual({'uuid': self.uuid}, res)
         eventlet.greenthread.sleep(functional.DEFAULT_SLEEP)
 
         status = self.client.get_status(self.uuid)
-        self.assertEqual({'finished': True, 'error': None},
-                         status)
+        self.check_status(status, finished=True)
         self.cli.port.create.assert_has_calls([port_create_call],
                                               any_order=True)
         self.assertFalse(get_mock.called)
@@ -116,8 +142,7 @@ class TestV1PythonAPI(functional.Base):
         self.assertEqual(202, res.status_code)
         self.assertEqual('', res.text)
         eventlet.greenthread.sleep(functional.DEFAULT_SLEEP)
-        self.assertEqual({'finished': True, 'error': None},
-                         status)
+        self.check_status(status, finished=True)
 
         self.cli.port.create.assert_has_calls([port_create_call,
                                                port_create_call],
@@ -137,7 +162,7 @@ class TestV1PythonAPI(functional.Base):
                                                               'reboot')
 
         status = self.client.get_status(self.uuid)
-        self.assertEqual({'finished': False, 'error': None}, status)
+        self.check_status(status, finished=False)
 
         res = self.client.abort(self.uuid)
         eventlet.greenthread.sleep(functional.DEFAULT_SLEEP)
@@ -146,8 +171,7 @@ class TestV1PythonAPI(functional.Base):
         self.assertEqual('', res.text)
 
         status = self.client.get_status(self.uuid)
-        self.assertEqual({'finished': True, 'error': 'Canceled by '
-                          'operator'}, status)
+        self.check_status(status, finished=True, error='Canceled by operator')
 
         # assert continue doesn't work after abort
         self.call_continue(self.data, expect_error=400)
@@ -247,7 +271,7 @@ class TestSimplePythonAPI(functional.Base):
                                                               'reboot')
 
         status = client.get_status(self.uuid)
-        self.assertEqual({'finished': False, 'error': None}, status)
+        self.check_status(status, finished=False)
 
         res = self.call_continue(self.data)
         self.assertEqual({'uuid': self.uuid}, res)
@@ -258,7 +282,7 @@ class TestSimplePythonAPI(functional.Base):
             node_uuid=self.uuid, address='11:22:33:44:55:66')
 
         status = client.get_status(self.uuid)
-        self.assertEqual({'finished': True, 'error': None}, status)
+        self.check_status(status, finished=True)
 
     def test_api_versions(self):
         minv, maxv = client.server_api_versions()
@@ -307,6 +331,14 @@ class BaseCLITest(functional.Base):
 
 
 class TestCLI(BaseCLITest):
+    def _fake_status(self, **kwargs):
+        # to remove the hidden fields
+        hidden_status_items = shell.StatusCommand.hidden_status_items
+        fake_status = super(TestCLI, self)._fake_status(**kwargs)
+        fake_status = dict(item for item in fake_status.items()
+                           if item[0] not in hidden_status_items)
+        return fake_status
+
     def test_cli_negative(self):
         err = self.run_cli('start', expect_error=True)
         self.assertIn('too few arguments', err)
@@ -327,7 +359,7 @@ class TestCLI(BaseCLITest):
                                                               'reboot')
 
         status = self.run_cli('status', self.uuid, parse_json=True)
-        self.assertEqual({'finished': False, 'error': None}, status)
+        self.check_status(status, finished=False)
 
         res = self.call_continue(self.data)
         self.assertEqual({'uuid': self.uuid}, res)
@@ -338,7 +370,7 @@ class TestCLI(BaseCLITest):
             node_uuid=self.uuid, address='11:22:33:44:55:66')
 
         status = self.run_cli('status', self.uuid, parse_json=True)
-        self.assertEqual({'finished': True, 'error': None}, status)
+        self.check_status(status, finished=True)
 
     def test_rules_api(self):
         res = self.run_cli('rule', 'list', parse_json=True)
@@ -381,4 +413,6 @@ class TestCLI(BaseCLITest):
 
 if __name__ == '__main__':
     with functional.mocked_server():
+        # Make links predictable
+        http._DEFAULT_URL = 'http://127.0.0.1:5050'
         unittest.main(verbosity=2)
